//===============================================================================
// Microsoft patterns & practices
// Mobile Client Software Factory - July 2006
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;

namespace Microsoft.Practices.Mobile.DisconnectedAgent
{
	public class RequestDispatchedEventArgs : EventArgs
	{
		private DispatchResultStatus result;
		private Request request;
		private Exception exception;

		/// <summary>
		/// It creates an RequestDispatchedEventArgs for the given request and result.
		/// It is used by the RequestDispatched event, fired during the dispatch process.
		/// </summary>
		/// <param name="request">Request tried to be dispatched.</param>
		/// <param name="result">Dispatch process result.</param>
		/// <param name="exception">The dispatch exception if dispatch failed</param>
		public RequestDispatchedEventArgs(Request request, DispatchResultStatus result, Exception exception)
		{
			this.request = request;
			this.result = result;
			this.exception = exception;
		}

		/// <summary>
		/// Request tried to be dispatched.
		/// </summary>
		public Request Request
		{
			get { return request; }
		}

		/// <summary>
		/// Result of the dispatch process.
		/// </summary>
		public DispatchResultStatus Result
		{
			get { return result; }
		}

		/// <summary>
		/// Dispatch Exception if an exception occured
		/// </summary>
		public Exception Exception
		{
			get { return exception; }
		}
	}
}
