//===============================================================================
// Microsoft patterns & practices
// Mobile Client Software Factory - July 2006
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.Mobile.EndpointCatalog;
using System.Web.Services.Protocols;
using System.Net;

namespace Microsoft.Practices.Mobile.DisconnectedAgent
{
	/// <summary>
	/// This class is the default concrete implementation of IProxyFactory.
	/// It uses the OnlineProxyType property of the Request to know the expected type for the proxy object.
	/// Additionaly it sets the URL and Credentials properties in the proxy object. 
	/// To set these properties, the factory uses the EndpointCatalog to get the right address and credentials
	/// for the current network name and the Request's endpoint.
	/// </summary>
	public class WebServiceProxyFactory : IProxyFactory
	{
		private IEndpointCatalog catalog;
        private IWSCredentialService credentialSvc;

		/// <summary>
		/// Constructor which sets the IEndpointCatalog to get credentials and address for endpoints.
		/// </summary>
		/// <param name="catalog">
		///		Endpoint catalog containing addresses and credentials account name for each endpoint and network names.
		/// </param>
        /// <param name="credentialSvc">
        ///		WebService Credential Service which provides credential for a given account name.
        /// </param>
        public WebServiceProxyFactory(IEndpointCatalog catalog, IWSCredentialService credentialSvc)
		{
			this.catalog = catalog;
            this.credentialSvc = credentialSvc;
		}

		/// <summary>
		/// This method creates the proxy object and set the URL and Credentials properties.
		/// </summary>
		/// <param name="request">Request to be dispatched.</param>
		/// <param name="networkName">Current network name.</param>
		/// <returns>The proxy object which has been created.</returns>
		public virtual object GetOnlineProxy(Request request, string networkName)
		{
			Guard.ArgumentNotNull(request, "request");
			SoapHttpClientProtocol proxy = (SoapHttpClientProtocol)Activator.CreateInstance(request.OnlineProxyType);
            NetworkCredential credentials = credentialSvc.GetCredential(
                catalog.GetAccountNameForEndpoint(request.Endpoint, networkName));
            if (credentials != null)
                proxy.Credentials = credentials;
			proxy.Url = catalog.GetAddressForEndpoint(request.Endpoint, networkName);
			return proxy;
		}
	}
}
