//===============================================================================
// Microsoft patterns & practices
// Mobile Client Software Factory - July 2006
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using Microsoft.Practices.Mobile.EndpointCatalog;

namespace Microsoft.Practices.Mobile.DisconnectedAgent.Tests.Mocks
{
	public class MockEndpointCatalog : IEndpointCatalog
	{
		public Dictionary<string, MockEndpoint> Endpoints = new Dictionary<string, MockEndpoint>();

		public bool AddressExistsForEndpoint(string endpointName, string networkName)
		{
			if (Endpoints.ContainsKey(endpointName))
			{
				return Endpoints[endpointName].ContainsConfigForNetwork(networkName);
			}
			return false;
		}

		public string GetAccountNameForEndpoint(string endpointName, string networkName)
		{
			if (Endpoints.ContainsKey(endpointName))
			{
				return Endpoints[endpointName].GetAccountNameForNetwork(networkName);
			}
			else
				throw new KeyNotFoundException("Endpoint doesn't exist in the catalog.");
		}

		public string GetAddressForEndpoint(string endpointName, string networkName)
		{
			if (Endpoints.ContainsKey(endpointName))
			{
				return Endpoints[endpointName].GetAddressForNetwork(networkName);
			}
			else
				throw new KeyNotFoundException("Endpoint doesn't exist in the catalog.");
		}

		public int Count
		{
			get { throw new Exception("The method or operation is not implemented."); }
		}

		public void Save(System.IO.TextWriter textWriter)
		{
			throw new Exception("The method or operation is not implemented.");
		}

		public void Load(System.IO.TextReader textReader)
		{
			throw new Exception("The method or operation is not implemented.");
		}

		public void SetEndpoint(Endpoint endpoint)
		{
			throw new Exception("The method or operation is not implemented.");
		}

		public void RemoveEndpoint(string endpointName)
		{
			throw new Exception("The method or operation is not implemented.");
		}

		public bool EndpointExists(string endpointName)
		{
			return (endpointName != "MissingEndpoint");
		}

		public void RemoveCredential(string endpointName, string networkName)
		{
			throw new Exception("The method or operation is not implemented.");
		}

		public void SetCredentialForEndpoint(string endpointName, string networkName, NetworkCredential credential)
		{
			throw new Exception("The method or operation is not implemented.");
		}
	}

	public class MockEndpoint
	{
		private MockEndpointConfig defaultConfig;
		private string name;

		public MockEndpointConfig Default
		{
			get { return defaultConfig; }
			set { defaultConfig = value; }
		}

		public string Name
		{
			get { return name; }
			set { name = value; }
		}


		public MockEndpoint(string name)
		{
			this.name = name;
		}

		public bool ContainsConfigForNetwork(string networkName)
		{
			if (networkName == "NotForThisNetwork") 
				return false;
			else
				return (this.Default != null);
		}

		public string GetAccountNameForNetwork(string networkName)
		{
			if (networkName == "NotForThisNetwork")
				return null;
			else
				return this.Default.AccountName;
		}

		public string GetAddressForNetwork(string networkName)
		{
			if (networkName == "NotForThisNetwork")
			{
				throw new KeyNotFoundException("There is not address for this Endpoint/Network");
			}
			else
				return this.Default.Address;
		}
	}

	public class MockEndpointConfig
	{
		public MockEndpointConfig (string adress)
		{
			this.address = adress;
		}

        private string accountName = "UserAndPasswordAccount";
		private string address;

		public string Address
		{
			get { return address; }
			set { address = value; }
		}


		public string AccountName
		{
			get { return accountName; }
			set { accountName = value; }
		}
	}

	
}
