//===============================================================================
// Microsoft patterns & practices
// Mobile Client Software Factory - July 2006
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.Mobile.DataAccess;
using Microsoft.Practices.TestDataAccess;
using System.Data.Common;
using System.Data.SqlServerCe;
using System.Data;

namespace Microsoft.Practices.Mobile.DisconnectedAgent.Tests
{
	public class MockQueueDatabase : Database, ISqlDatabase
	{
		private MockRequestTable table = new MockRequestTable();
		public bool RequestTablePresent = false;
		public string TableName;

		public MockQueueDatabase() : base("MockConnectionString", new MockProvider())
		{
		}

		public override bool TableExists(string tableName)
		{
			return RequestTablePresent;
		}

		public override int ExecuteNonQuery(string sqlCommand, params DbParameter[] parameters)
		{
			MockSql info = new MockSql(sqlCommand);

			if (sqlCommand.StartsWith("CREATE TABLE " + TableName))
				RequestTablePresent = true;

			if (sqlCommand.StartsWith("INSERT "))
				MockSql.AddRow(this.table, parameters);

			if (sqlCommand.StartsWith("UPDATE "))
				table.Rows[0]["Status"] = 1;

			if (sqlCommand.StartsWith("DELETE FROM "))
			{
				DataRow row = table.Rows.Find(parameters[0].Value);
				if (row != null)
					table.Rows.Remove(row);
			}

			return 0;
		}

		public override object ExecuteScalar(string sqlCommand, params DbParameter[] parameters)
		{
			MockSql info = new MockSql(sqlCommand);
			if (sqlCommand.StartsWith("SELECT Count"))
			{
				if (info.Where != null && info.Where.Length > 0)
				{
					foreach(DbParameter param in parameters)
					info.Where = info.Where.Replace(param.ParameterName, "'" + param.Value + "'");

					DataView view = new DataView(table, info.Where, info.OrderBy, DataViewRowState.CurrentRows);
					return view.Count;
				}
				else
					return table.Rows.Count;
			}

			return 0;
		}

		public override DbDataReader ExecuteReader(string sqlCommand, params DbParameter[] parameters)
		{
			MockSql info = new MockSql(sqlCommand);

			if (info.Where != null && info.Where.Length > 0)
			{
				foreach (DbParameter parameter in parameters)
				{
					if (parameter.DbType == DbType.String)
						info.Where = info.Where.Replace("= " + parameter.ParameterName, "= '" + parameter.Value.ToString() + "'");
					else if (parameter.DbType == DbType.Guid)
						info.Where = info.Where.Replace("= " + parameter.ParameterName, "= '" + parameter.Value.ToString() + "'");
					else
						info.Where = info.Where.Replace(">= " + parameter.ParameterName, ">= " + parameter.Value.ToString());
				}
				DataView view = new DataView(table, info.Where, info.OrderBy, DataViewRowState.CurrentRows);
				return view.ToTable().CreateDataReader();
			}
			else
				return table.CreateDataReader();
		}

		public DbParameter CreateParameter(string name, SqlDbType type, int size, object value)
		{
			return base.CreateParameter(name, DbType.String, size, value);
		}

		public SqlCeResultSet ExecuteResultSet(SqlCeCommand command, ResultSetOptions options, params DbParameter[] parameters)
		{
			return null;
		}

		public SqlCeResultSet ExecuteResultSet(string sqlCommand, ResultSetOptions options, params DbParameter[] parameters)
		{
			return null;
		}

		public override string BuildParameterName(string name)
		{
			return name;
		}
	}
}
