//===============================================================================
// Microsoft patterns & practices
// Mobile Client Software Factory - July 2006
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.Mobile.Configuration;

namespace Microsoft.Practices.Mobile.EndpointCatalog.Configuration
{
	/// <summary>
	/// Helper class to get an endpoint from the configuration.
	/// </summary>
	public class EndpointItemElement : ConfigurationElement
	{
		/// <summary>
		/// Endpoint name for the configuration.
		/// </summary>
		[ConfigurationProperty("Name", IsKey = true, IsRequired = true)]
		public string Name
		{
			get { return (string)this["Name"]; }
			set { this["Name"] = value;	}
		}

		/// <summary>
		/// Default Address for the endpoint.
		/// </summary>
		[ConfigurationProperty("Address", IsKey = false, IsRequired = false)]
		public string Address
		{
			get	{ return (string)this["Address"]; }
			set	{ this["Address"] = value; }
		}

        /// <summary>
        /// Default AccountName for the the endpoint
        /// </summary>
        [ConfigurationProperty("AccountName", IsKey = false, IsRequired = false)]
        public string AccountName
        {
            get { return (string)this["AccountName"]; }
            set { this["AccountName"] = value; }
        }

		/// <summary>
		/// Networks section with the configuration overrides for specific networks.
		/// </summary>
		[ConfigurationProperty("NetworkItems", IsRequired = false)]
		public NetworkElementCollection Networks
		{
			get { return (NetworkElementCollection)this["NetworkItems"]; }
			set { this["NetworkItems"] = value; }
		}

	}
}
