//===============================================================================
// Microsoft patterns & practices
// Mobile Client Software Factory - July 2006
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Net;
using Microsoft.Practices.TestUtilities;

namespace Microsoft.Practices.Mobile.EndpointCatalog.Tests
{
	[TestClass]
	public class EndpointConfigFixture
	{
		[TestMethod]
		public void CreationWithAddressKeepsTheGivenAddress()
		{
			string expected = "Address";
			EndpointConfig ec = new EndpointConfig(expected);

			Assert.AreEqual(expected, ec.Address);
		}

		[TestMethod]
		public void CreationWithAddressAndAccountNameKeepsValues()
		{
			EndpointConfig ec = new EndpointConfig("A", "account");

			Assert.AreEqual("account", ec.AccountName);
			Assert.AreEqual("A", ec.Address);
		}

		[TestMethod]
		public void CreationWithNullAddressThrowsArgumentNullException()
		{
			ExtendedAssert.Throws<ArgumentNullException>(
						delegate { EndpointConfig ec = new EndpointConfig(null); });
		}

		[TestMethod]
		public void CreationWithNullAccountCreatesNullAccountName()
		{
			EndpointConfig ec = new EndpointConfig("A", null);
			Assert.IsNull(ec.AccountName);
		}

		[TestMethod]
		public void ModifiedAccountNameReplacePreviousOne()
		{
            string account = "account";
    		EndpointConfig ec = new EndpointConfig("A", account);
			string expected = "NewAccount";
			ec.AccountName = expected;

			Assert.AreEqual(expected, ec.AccountName);
		}

	}
}
