//===============================================================================
// Microsoft patterns & practices
// Mobile Client Software Factory - July 2006
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using System.Configuration;
using System.CodeDom;
using Microsoft.Practices.Mobile.ObjectBuilder;
using System.Globalization;
using Microsoft.FxCop.Sdk;

namespace Microsoft.Practices.ObjectBuilder.ObGen
{
	public abstract class DependencyPolicyGenerator : PolicyGenerator
	{
		static TypeNode DependencyAttributeType = AssemblyNode.GetAssembly(typeof(DependencyAttribute)
			.Assembly.ManifestModule.FullyQualifiedName)
			.GetType(Identifier.For(typeof(DependencyAttribute).Namespace), Identifier.For(typeof(DependencyAttribute).Name));

		protected AttributeNode GetInjectionAttribute(AttributeList attributes, bool createDefault)
		{
			foreach (AttributeNode attribute in attributes)
			{
				if (typeof(ParameterAttribute).IsAssignableFrom(attribute.Type..GetRuntimeType()))
				{
					return attribute;
				}
			}

			if (createDefault)
			{
				// ReflectionStrategy automatically adds a DependencyAttribute if no ParameterAttribute is defined.
				AttributeNode dependencyAttribute = new AttributeNode(new MemberBinding(null, DependencyAttributeType.GetConstructors()[0]), new ExpressionList());
				dependencyAttribute.Type = DependencyAttributeType;

				return dependencyAttribute;
			}

			return null;
		}
	}
}
