//===============================================================================
// Microsoft patterns & practices
// Mobile Client Software Factory - July 2006
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Configuration;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;

namespace Microsoft.Practices.ObjectBuilder.ObGen
{
	/// <summary>
	/// Base policy configuration element for policies that implement their own <see cref="CreatePolicyGenerator"/> method.
	/// </summary>
	/// <remarks>
	/// Cannot be abstract because of the configuration infrastructure.
	/// </remarks>
	public class PolicyGeneratorFactoryElement : NameTypeConfigurationElement
	{
		public virtual IPolicyGenerator CreatePolicyGenerator()
		{
			throw new NotImplementedException();
		}
	}

	/// <summary>
	/// Base policy configuration element for policies that have a parameterless constructor and so don't require 
	/// overriding this class default <see cref="CreatePolicyGenerator"/> method implementation.
	/// </summary>
	public class PolicyGeneratorFactoryElement<TPolicy> : PolicyGeneratorFactoryElement
		where TPolicy : IPolicyGenerator, new()
	{
		public override IPolicyGenerator CreatePolicyGenerator()
		{
			return new TPolicy();
		}
	}
}
