//===============================================================================
// Microsoft patterns & practices
// Mobile Client Software Factory - July 2006
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;

namespace Microsoft.Practices.Mobile.ObjectBuilder
{
	/// <summary>
	/// Attribute that represents dependency injection members, whose value at 
	/// build time will be determined by the <see cref="IParameter"/> returned 
	/// from the attribute <see cref="CreateParameter"/> factory method.
	/// </summary>
	public abstract class ParameterAttribute : Attribute
	{
		/// <summary>
		/// Initializes an instance of the <see cref="ParameterAttribute"/> class.
		/// </summary>
		protected ParameterAttribute() { }

		/// <summary>
		/// Creates a parameter for use with various <see cref="IBuilderPolicy"/> implementations 
		/// that can process <see cref="IParameter"/>s.
		/// </summary>
		/// <param name="memberType">The type of the annotated member, such as a property or a 
		/// constructor parameter.</param>
		/// <returns>The parameter instance that knows how to retrieve a value for the dependency.</returns>
		public abstract IParameter CreateParameter(Type memberType);
	}
}
