//===============================================================================
// Microsoft patterns & practices
// Mobile Client Software Factory - July 2006
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Reflection;

namespace Microsoft.Practices.Mobile.ObjectBuilder
{
	/// <summary>
	/// Simple default creation policy which selects the first public constructor
	/// of an object, using the builder to resolve/create any parameters the
	/// constructor requires.
	/// </summary>
	public class DefaultCreationPolicy : ICreationPolicy
	{
		/// <summary>
		/// See <see cref="ICreationPolicy.SelectConstructor"/> for more information.
		/// </summary>
		public ConstructorInfo SelectConstructor(IBuilderContext context, Type type, string id)
		{
			Guard.ArgumentNotNull(type, "type");

			ConstructorInfo[] constructors = type.GetConstructors();

			if (constructors.Length > 0)
				return constructors[0];

			return null;
		}

		/// <summary>
		/// See <see cref="ICreationPolicy.GetParameters"/> for more information.
		/// </summary>
		public object[] GetParameters(IBuilderContext context, Type type, string id, ConstructorInfo constructor)
		{
			Guard.ArgumentNotNull(context, "context");
			Guard.ArgumentNotNull(constructor, "constructor");

			ParameterInfo[] parms = constructor.GetParameters();
			object[] parmsValueArray = new object[parms.Length];

			for (int i = 0; i < parms.Length; ++i)
				parmsValueArray[i] = context.HeadOfChain.BuildUp(context, parms[i].ParameterType, null, id);

			return parmsValueArray;
		}
	}
}
