//===============================================================================
// Microsoft patterns & practices
// Mobile Client Software Factory - July 2006
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Reflection;

namespace Microsoft.Practices.Mobile.ObjectBuilder
{
	/// <summary>
	/// Interface used by the <see cref="ReflectionStrategy{T}"/> base class to encapsulate the information
	/// required from members that use the strategy. This interface is required because direct access to
	/// the <see cref="MemberInfo"/> object may not give the desired results.
	/// </summary>
	public interface IReflectionMemberInfo<TMemberInfo>
	{
		/// <summary>
		/// Gets the original member info object.
		/// </summary>
		TMemberInfo MemberInfo { get; }

		/// <summary>
		/// Gets the name of the member.
		/// </summary>
		string Name { get; }

		/// <summary>
		/// Gets the custom attributes of the member.
		/// </summary>
		/// <returns></returns>
		object[] GetCustomAttributes(Type attributeType, bool inherit);

		/// <summary>
		/// Gets the parameters to be passed to the member.
		/// </summary>
		/// <returns></returns>
		ParameterInfo[] GetParameters();
	}
}
