//===============================================================================
// Microsoft patterns & practices
// Mobile Client Software Factory - July 2006
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;

namespace Microsoft.Practices.Mobile.ObjectBuilder
{
	/// <summary>
	/// An implementation helper class for <see cref="IParameter"/> which can be used
	/// when you know the type of the parameter value ahead of time.
	/// </summary>
	public abstract class KnownTypeParameter : IParameter
	{
		/// <summary>
		/// The parameter type.
		/// </summary>
		protected Type type;

		/// <summary>
		/// Initializes a new instance of the <see cref="KnownTypeParameter"/> class
		/// using the given type.
		/// </summary>
		/// <param name="type">The parameter type.</param>
		protected KnownTypeParameter(Type type)
		{
			this.type = type;
		}

		/// <summary>
		/// See <see cref="IParameter.GetParameterType"/> for more information.
		/// </summary>
		public Type GetParameterType(IBuilderContext context)
		{
			return type;
		}

		/// <summary>
		/// Abstract for <see cref="IParameter.GetValue"/>. Derived classes are required
		/// to provide the implemenation for providing the value itself.
		/// </summary>
		/// <param name="context">The builder context.</param>
		/// <returns>The parameter value.</returns>
		public abstract object GetValue(IBuilderContext context);
	}
}